// fractiontutor.java
// David C. Hitchcock
// 1 November 2001
// This program presents the user with a fraction addition problem.
// If the answer is incorrect, it attempts to guess what the user 
// did wrong, and to give appropriate instruction.
// It was originally written in Python as a class project for 
// CS-335 Artificial Intelligence at Northeastern Illinois
// University.
// It is written for my former GED math students at Wilbur Wright
// College, as well as for the students at the other City Colleges
// of Chicago, and everywhere.
//
// Because it is an applet, it has no "main" method.  

//import javax.swing.*;  
import java.awt.*;
import java.awt.event.*;
//import Rational;// Needed if Rational is in a different directory
import java.util.Random;
import java.applet.Applet;

public class fractiontutor extends Applet {

    public void init(){

	Settings Panel1;        // A class defined below
	Calculator Panel2;      // A class defined below
	
	setLayout(new BorderLayout());
	
	Panel1 = new Settings();
	add(Panel1, BorderLayout.WEST);

	Panel2 = new Calculator();
	add(Panel2, BorderLayout.CENTER);
    }
}

class Settings extends Panel implements 
		     ItemListener, ActionListener {
    Label typelabel;
    CheckboxGroup fractiontype;
    Checkbox whole;
    Checkbox proper;
    Checkbox improper;
    static String fractype = new String("proper");
    Label displaymodelabel;
    CheckboxGroup displaymode;
    Checkbox unreduced;
    Checkbox reduced;
    static boolean dispreduced = true;
    Label maxvallabel;
    static TextField maxval;
    static String maxstring;
    static int max = 10;
    Label functionlabel;
    CheckboxGroup function;
    Checkbox tutorbutton;
    Checkbox helper; 
    Checkbox calculator;

    static boolean userproblem = false;
    static boolean tutor = true;

    Settings(){
	setLayout(new GridLayout(13, 1));
	add(typelabel = new Label("Fraction Type"));
	fractiontype = new CheckboxGroup();
	add(whole = new Checkbox("Whole Numbers", 
				 false, fractiontype));
	whole.addItemListener(this);
	add(proper = new Checkbox("Proper Fractions", 
				  true, fractiontype));
	proper.addItemListener(this);
	add(improper = new Checkbox("Improper Fractions",
				    false, fractiontype));
	improper.addItemListener(this);
        add(displaymodelabel = new Label("Display Mode"));
	displaymode = new CheckboxGroup();
	add(reduced = new Checkbox("Lowest Terms", 
				   true, displaymode));
	reduced.addItemListener(this);
	add(unreduced = new Checkbox("Not Reduced",
				     false, displaymode));
	unreduced.addItemListener(this);
	add(maxvallabel = new Label("Maximum Value"));
	add(maxval = new TextField("10", 6));
	maxval.addActionListener(this);
	add(functionlabel = new Label("Function"));
	function = new CheckboxGroup();
	add(tutorbutton = new Checkbox("Tutor", true, function));
	tutorbutton.addItemListener(this);
	add(helper = new Checkbox("Helper", false, function));
	helper.addItemListener(this);
	add(calculator = new Checkbox("Calculator", 
				      false, function));
	calculator.addItemListener(this);
    }

    public void itemStateChanged(ItemEvent e) {
	if (e.getItemSelectable() == whole) 
	    fractype = "whole";
	else if (e.getItemSelectable() == proper)
	    fractype = "proper";
	else if (e.getItemSelectable() == improper)
	    fractype = "improper";
	else if (e.getItemSelectable() == reduced) 
	    dispreduced = true;
	else if (e.getItemSelectable() == unreduced)
	    dispreduced = false;
	else if (e.getItemSelectable() == tutorbutton){
	    tutor = true;
	    userproblem = false; 
	    Calculator.setTutorMode();
	}
	else if (e.getItemSelectable() == helper){
	    tutor = true;
	    userproblem = true; 
	    Calculator.setHelperMode();
	    Calculator.instructions.setText(Calculator.helperString);
	}	    
	else if (e.getItemSelectable() == calculator) {
	    tutor = false;
	    userproblem = true; 
	    Calculator.setHelperMode();
	    Calculator.instructions.setText(
			      Calculator.calculatorString);    
	}
    }

        public void actionPerformed(ActionEvent e) {
    // Change maximum when maxval is changed. 
	if(e.getSource() == maxval) {
	    maxstring = maxval.getText();
	    //Check that the String is not empty.
	    if (maxstring.length() == 0) {
		maxval.setText("Error");
		Calculator.instructions.setText("Maximum must be a positive whole number.");
		return;
	    }
	    // Check that the String contains only digits.
	    for (int i = 0; i < maxstring.length(); i++) { 
		char next = maxstring.charAt(i);
		if (!(next == '0' || next == '1' || next == '2' || 
		    next == '3'	|| next == '4' || next == '5' || 
	            next == '6' || next == '7' || next == '8' || 
		      next == '9')) {
			  maxval.setText("Error");
			  Calculator.instructions.setText("Maximum must be a positive whole number.");
			  return;
		}
	    }
	    max = Integer.parseInt(maxstring);
				   //maxval.getText());
	}
    }

}


class Calculator extends Panel implements ActionListener {

    Panel calcpanel;
    static TextField addend1; //The following elements, which make
    static Rational rat1 = null; // up a claculator, will be added
    Label plus;   // to calcpanel
    static TextField addend2;
    static Rational rat2 = null;
    static Button equals;
    static TextField sum;
    static Rational total = null;

    // Display instructions on using the program
    // and on adding fractions
    static TextArea instructions;  

    // The method dotutor() is a finite state machine.  Each time
    // it is invoked, it will do things, then update tutorstate,
    // then return in order to accept further input from the user.
    // tutorstate is declared here rather than in dotutor() so 
    // that it won't go out of existence when dotutor() returns.
    private int tutorstate = 0;  

    Calculator(){
	setLayout(new BorderLayout());
	calcpanel = new Panel();
	calcpanel.setLayout(new GridLayout(1,5));
	addend1 = new TextField(6);
	calcpanel.add(addend1);
	calcpanel.add(plus = new Label("   +  "));
	addend2 = new TextField(6);
	calcpanel.add(addend2);
	calcpanel.add(equals = new Button("="));
	equals.addActionListener(this);
	sum = new TextField(6);
	calcpanel.add(sum);
	sum.addActionListener(this);

	add(calcpanel, BorderLayout.NORTH);

	instructions = new TextArea("Hello!\n   Hello!!\n\n      Hello!!!", 15, 40);
        instructions.setEditable(false);
	// This  needs a fixed font to properly display
	// fractions, which have the numerator and denominator
	// displayed on different lines.  
 	instructions.setFont(new Font("Monospaced", Font.PLAIN, 12));
	// We get a scrollbar by putting the textfield 
	// in a JScrollPane 
        ScrollPane areaScrollPane = new ScrollPane();
	areaScrollPane.add(instructions);
	add(areaScrollPane, BorderLayout.CENTER);

	rat1 = new Rational();
	rat2 = new Rational();
	total = new Rational();
	setTutorMode();
    }

    // This function is called by Settings or the constructor to 
    public static void setTutorMode(){
	rat1.setRandom(Settings.max, 
		       Settings.fractype, Settings.dispreduced);
	addend1.setText(rat1.tostring());
	addend1.setEditable(false);
	rat2.setRandom(Settings.max, 
		       Settings.fractype, Settings.dispreduced);
	addend2.setText(rat2.tostring());
	addend2.setEditable(false);
	sum.setText("");
	instructions.setText(tutorString);
    }

    public static String tutorString 
	= new String("This is the Tutor mode.  The computer\ngives you a problem, and you type the\nanswer, then hit 'Enter.' The computer\nwill tell you if your answer is right or\nwrong.  It will also tutor you\nif you make a mistake.\n\nTo enter the answer, first enter the\nnumerator (the top number), followed by\na '/', then the denominator (the bottom\nnumber), then hit 'Enter'.  If your\nanswer is a whole number, type the\nnumber, then hit 'Enter'.\n");

    public static String helperString 
	= new String("This is the Helper mode.  You enter a\nproblem on which you want help, then\ntype the answer and hit 'Enter.' The\ncomputer will tell you if your answer is\nright or wrong.  It will also tutor you\nif you make a mistake.\n\nTo enter the numbers you want to add, or\nthe answer, first enter the numerator\n(the top number), followed by a '/',\nthen the denominator (the bottom number).\nTo enter a whole number, just type the\nnumber. \nAfter you enter the answer, hit 'Enter'.\n");

    public static String calculatorString 
	= new String("This is the Calculator mode.  You enter a\na problem, and click on the '=' and\nthe computer will give you the answer.\n\nTo enter a number, first enter the\nnumerator (the top number), followed by\na '/', then the denominator (the bottom\nnumber). \nIf your answer is a whole number, type\nthe number, then hit 'Enter'.\n");

    // Called by Settings for helper or calculator mode
    public static void setHelperMode(){
	try {
	rat1.setFromString("0");
	rat2.setFromString("0");
	} catch (Rational.BadStringException e) {
	    instructions.append("Error " + e.badstring);
	    sum.setText("Error!");
	    return;
	}
	addend1.setText("");
	addend1.setEditable(true);
	addend2.setText("");
	addend2.setEditable(true);
	sum.setText("");
    }

    public void actionPerformed(ActionEvent a) {
	if(a.getSource() == equals) {
	    // The following statement can be used for debugging
	    //	    instructions.append("\nType= " + Settings.fractype + "  Reduced = " + String.valueOf(Settings.dispreduced) + "  Userproblem = " + String.valueOf(Settings.userproblem) + "\nTutor = " + String.valueOf(Settings.tutor) + "  Max = " + String.valueOf(Settings.max) + "\n");
	    if (Settings.tutor == false) { // Calculator mode
		try {
		rat1.setFromString(addend1.getText()); 
		rat2.setFromString(addend2.getText());
		instructions.append("addend1 = " + addend1.getText() + "  addend2 = " + addend2.getText() + "\n");
		} catch (Rational.BadStringException e) {
		    instructions.append("Error " + e.badstring);
		    sum.setText("Error!");
		    return;
		}
		total= rat1.plus(rat2);
		sum.setText(total.tostring());
		instructions.append("sum = " + total.tostring() + "\n");
	    }
	    else  // Settings.tutor == true, tutor or helper mode
		dotutor();
	}
	// Tutor or Helper mode
	else if(a.getSource() == sum && Settings.tutor == true)
	    dotutor();
    }

    // Carries out the tutoring function for Tutor and Helper modes
    // Generates new problems in Tutor mode
    public void dotutor(){
	// a finite state machine, with state stored in tutorstate.
	// The states are:
	// 1: Last answer was correct, on another click a new
	//    problem will be presented. Used in tutor mode. 
	// 0: dotutor() sets tutorstate to 0 if the answer
	//    is correct in helper mode, or after supplying a
	//    new problem in tutor mode.
	// 2: The previous answer needed to be reduced.
	// 3: The previous answer was the result of adding 
	//    numerators and denominators.
	// 4: The previous answer was the result of subtracting
	// 5: The previous answer was the product of the addends
	// 6: The previous answer was the quotient
	// 7: The previous answer was the result of subtracting
	//    numerators and denominators. 
	// 8: There was some other mistake.

	if (tutorstate == 1) {
	    rat1.setRandom(Settings.max, Settings.fractype, 
			   Settings.dispreduced);
	    addend1.setText(rat1.tostring());
	    rat2.setRandom(Settings.max, Settings.fractype, 
			   Settings.dispreduced);
	    addend2.setText(rat2.tostring());
	    sum.setText("");
	    instructions.setText("To enter the answer, first enter the\nnumerator (the top number), followed by\na '/', then the denominator (the bottom\nnumber), then hit 'Enter'.  If your\nanswer is a whole number, type the\nnumber, then hit 'Enter'.\n");
	    tutorstate = 0;
	    return;
	 }

	// test is used to compare the given answer to the correct
	// answer, and if it doesn't match, to various answers
	// answers resulting from incorrect algorithms.
	Rational test = new Rational();

	// If we are in Helper mode, set rat1 and rat2 
	if (Settings.userproblem) {
	    try {
		rat1.setFromString(addend1.getText()); 
		rat2.setFromString(addend2.getText());
		} catch (Rational.BadStringException e) {
		    instructions.append("Error\n" + e.badstring);
		    return;
		}
	}

	// Read the answer given by the user
	try {
	total.setFromString(sum.getText()); 
	} catch (Rational.BadStringException e) {
	    instructions.append("Error\n" + e.badstring);
	    return;
	}

	// answer is used to test whether the answer needs
	// to be reduced.
	Rational answer = new Rational();
	try {
	answer.setFromString(sum.getText());
	} catch (Rational.BadStringException e) {
	instructions.append("Error\n" + e.badstring);
	return;
	}
	answer.reduce(); // The given answer in lowest terms

	// Compute the correct answer
	test = rat1.plus(rat2);
	// See if the given answer is correct
	if (test.isEqual(total)) { 
	    instructions.setText("\nRight!\n");
	    newproblem();
	}

	else if (test.isEqual(answer)) {
	    // The answer is correct, except it's not lowest terms
	    if (tutorstate == 2) { // Same mistake twice
		int factor;
		factor = total.numeratorof() / test.numeratorof();
		instruct(total.numeratorof(), 
			 total.denominatorof(), factor);
		newproblem();
	    }
	    else {
		instructions.setText("See if you can reduce it!\n");
		tutorstate = 2;
	    }
	}
	else if (total.isEquivalent(rat1.addboth(rat2))){
	    if (tutorstate == 3 || tutorstate == 7){
		instruct(rat1.numeratorof(), rat1.denominatorof(),
			 rat2.numeratorof(), rat2.denominatorof(),
			 test.denominatorof());
		newproblem();
	    }
	    else {
		instructions.setText("You seem to have added both the\nnumerator and the denominator.\nTry again!\n");
		tutorstate = 3;
	    }
	}
	else if (total.isEquivalent(rat1.minus(rat2))) {
	    if (tutorstate == 4) {
		instruct(rat1.numeratorof(), rat1.denominatorof(),
			 rat2.numeratorof(), rat2.denominatorof(),
			 test.denominatorof());
		newproblem();
	    }
	    else {
		instructions.setText("You seem to have subtracted\nTry again!\n");
		tutorstate = 4;
	    }
	}
	else if (total.isEquivalent(rat1.times(rat2))) {
	    if (tutorstate == 5) {
		instruct(rat1.numeratorof(), rat1.denominatorof(),
			 rat2.numeratorof(), rat2.denominatorof(),
			 test.denominatorof());
		newproblem();
	    }
	    else {
		instructions.setText("You seem to have multiplied\nTry again!\n");
		tutorstate = 5;
	    }
	}
	else if (total.isEquivalent(rat1.dividedby(rat2))){
	    if (tutorstate == 6) {
		instruct(rat1.numeratorof(), rat1.denominatorof(),
			 rat2.numeratorof(), rat2.denominatorof(),
			 test.denominatorof());
		newproblem();
	    }
	    else {
		instructions.setText("You seem to have divided\nTry again!\n");
		tutorstate = 6;
	    }
	}
	else if (total.isEquivalent(rat1.subtractboth(rat2))) {
	    if (tutorstate == 7) {
		instruct(rat1.numeratorof(), rat1.denominatorof(),
			 rat2.numeratorof(), rat2.denominatorof(),
			 test.denominatorof());
		newproblem();
	    }
	    else {
		instructions.setText("You seem to have subtracted both\nthe numerator and denominator.\nTry again!\n");
		tutorstate = 7;
	    }
	}
	else {
	    if (tutorstate == 8) {
		instruct();
		newproblem();
	    }
	    else {
		instructions.setText("Did you copy the numbers correctly?\nDid you copy your answer correctly?\nIf so, you seem to have an error in your arithmetic.\nTry again!\n");				         tutorstate = 8;
	    }
	}
     }


// Gives instruction for errortype = 'add', 'reduce', or 'mistake'
// Shows the student how to work the problem they have done
// incorrectly twice.
    public void instruct(int top1, int bottom1, int top2, 
			     int bottom2, int ansdenom) {
	instructions.setText("Addition Instructions:\n");
	int factor;
	int numerator;
	int denominator;
	if (bottom1 == bottom2){
	    numerator = top1 + top2;
	    denominator = bottom1;
	    factor = bottom1 / ansdenom;
	    instructions.append(
	       "We already have a common denominator,\n" + 
	       "so we add the numerators (top numbers):\n" +
	       String.valueOf(top1) + " + " + String.valueOf(top2)
	       + " = " + String.valueOf(numerator) +
	       "\nand we keep the same denominator\n" + 
	       "(bottom number), so we get " + 
	       String.valueOf(numerator) + "/" + 
	       String.valueOf(denominator) + "\n");
	}
	else {
	    int n1 = top1 * bottom2;
	    denominator = bottom1 * bottom2;
	    int n2 = top2 * bottom1;
	    numerator = n1 + n2;
	    factor = denominator / ansdenom; 
	    instructions.append(
	       "First we need to find a common denominator\n" +
	       "(bottom number).  One way to do this is\n" +
	       "to multiply both the numerator (top\n" +
	       "number) and denominator of the first\n" +
	       "fraction by the denominator of the second:\n " + 
	       String.valueOf(top1) + " x " + 
	       String.valueOf(bottom2) + "          " + 
	       String.valueOf(n1) + 
	       "\n------------- = ------\n " + 
	       String.valueOf(bottom1) + " x " + 
	       String.valueOf(bottom2) + "          " + 
	       String.valueOf(denominator) + 
	       "\nand multiply both the numerator and\n" +
	       "denominator of the second fraction by\n" +
	       "the denominator of the first:\n " + 
	       String.valueOf(top2) + " x " + 
	       String.valueOf(bottom1) + "          " + 
	       String.valueOf(n2) + 
	       "\n------------- = ------\n " + 
	       String.valueOf(bottom2) + " x " + 
	       String.valueOf(bottom1) + "          " + 
	       String.valueOf(denominator) + 
	       "\nNow we have a common denominator, " + 
	       String.valueOf(denominator) + 
	       "\nso we add the numerators:\n  " + 
	       String.valueOf(n1) + " + " + String.valueOf(n2) + 
	       " = " + String.valueOf(numerator) +  
	       "\nand we keep the same denominator, so\n" + 
	       "we get " + String.valueOf(numerator) + "/" + 
	       String.valueOf(denominator) + "\n");
	}
	if (factor == 1)
	    instructions.append(
	        "It is in lowest terms, so that's the answer!\n");
	else {
	    instructions.append(
                "Now we need to reduce it to lowest terms.\n");
	    instruct(numerator, denominator, factor);
	}
     }

// Show the student how to reduce the fraction to lowest terms.
    public void instruct(int top, int bottom, int factor){
	instructions.append(
            "The numerator and denominator can both be\n" +
	    "divided by " + String.valueOf(factor) + 
	    " so we get:\n " + 
	    String.valueOf(top) + " / " + String.valueOf(factor) + 
	    "          " + String.valueOf(top/factor) +  
	    "\n------------- = ------\n " + 
	    String.valueOf(bottom) + " / " + 
	    String.valueOf(factor) + "          " + 
	    String.valueOf(bottom/factor) +  
	    "\n(Here the '/' means 'divided by')\n");
    }
	
    public void instruct() {
	instructions.setText(
	    "Did you copy the numbers correctly?\n" + 
	    "Did you copy your answer correctly?\n" +
	    "If so, you seem to have an error in your\n" +
	    "arithmetic.  You may want to review the\n" +
	    "addition and multiplication tables.\n");
    }
     
    public void newproblem() {
	if (Settings.userproblem) { // Helper mode
	    tutorstate = 0;
	    instructions.append("Enter another problem on which you want help, then\ntype the answer and hit 'Enter.' The\ncomputer will tell you if your answer is\nright or wrong.  It will also tutor you\nif you make a mistake.\n\nTo enter the numbers you want to add, or\nthe answer, first enter the numerator\n(the top number), followed by a '/',\nthen the denominator (the bottom number).\nTo enter a whole number, just type the\nnumber. \nAfter you enter the answer, hit 'Enter'.\n");
	    return;
	}
	else { // Tutor mode
	    tutorstate = 1;
	    instructions.append("Hit 'Enter' for another problem");
	    return;
	}
    }
}
